﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PlanetFactory : MonoBehaviour {

	public MainGameControllerBehaviour MainController;
	public PlanetBehaviour PlanetPrefab;

	// Used for texture shuffle bag
	private List<Sprite> _planetTextures;
	private List<Sprite> _planetShuffleBag;

	private int _maxPlanets = 5;

	// Spawning variables
	private float _spawnRate = 0f;
	private float _respawnTimer = 0.0f;
	private float _spawnZoneWidth = 0f;
	private float _spawnZoneHeight = 0f;
	private float _spawnMinX = 0f;

	private List<PlanetBehaviour> _activePlanets;
	private List<PlanetBehaviour> _tempPlanets;

	void Start() {
		_activePlanets = new List<PlanetBehaviour>();
		_tempPlanets = new List<PlanetBehaviour>();

		// Populate shuffle bag
		_planetTextures = new List<Sprite>();
		_planetShuffleBag = new List<Sprite>();
		_planetTextures.AddRange(Resources.LoadAll<Sprite>("Planets"));
		_planetShuffleBag.AddRange(_planetTextures);

		SetSpawnZone();
	}

	/*
	 * Sets the default spawn zone locations
	 */
	public void SetSpawnZone() {
		_spawnZoneWidth = MainController.mainCam.ScreenToWorldPoint(new Vector3(Screen.width*1.65f, 0f, 0f)).x;
		_spawnZoneHeight = MainController.mainCam.ScreenToWorldPoint(new Vector3(0f, Screen.height, 0f)).y + 5.5f;
		_spawnMinX = MainController.mainCam.ScreenToWorldPoint(new Vector3(10.0f, 0f, 0f)).x;
	}
	
	// Update is called once per frame
	public void UpdatePlanets (MainGameControllerBehaviour.GameState currentState) {
		// Don't create new planets during pause!
		if (currentState != MainGameControllerBehaviour.GameState.Paused) {
			GeneratePlanet();
		}

		_tempPlanets.Clear();
		_tempPlanets.AddRange(_activePlanets);

		// Update each active planet
		foreach(PlanetBehaviour planet in _tempPlanets) {
			planet.UpdatePlanet(currentState);

			// Remove planet if needed
			if(planet.ShouldBeDestroyed) {
				DestroyPlanet(planet);
			}
		}
	}

	/*
	 * Randomly generates a new planet
	 */
	private void GeneratePlanet() {
		_respawnTimer += Time.deltaTime;

		// Check if a new planet should be generated
		if ((_activePlanets.Count < _maxPlanets) && (_respawnTimer >= _spawnRate)) {

			// Randomize a new planet
			int randomStartX = Random.Range((int)(_spawnMinX), (int)(_spawnMinX+_spawnZoneWidth));

			PlanetBehaviour newPlanet = (PlanetBehaviour)Instantiate(PlanetPrefab, 
			                                                         new Vector3((float)randomStartX, _spawnZoneHeight, 0.0f),
			                                                         transform.rotation);

			SpriteRenderer planetSprite = ((SpriteRenderer)newPlanet.GetComponent("SpriteRenderer"));
			planetSprite.sprite = GetSpriteFromShuffleBag();

			_activePlanets.Add(newPlanet);

			// Reset spawn time functions
			_spawnRate = Random.Range(10f, 15f);
			_respawnTimer = 0.0f;
		}
	}

	/*
	 * Returns a random sprite from the shuffle bag and reshuffles if necessary
	 */
	private Sprite GetSpriteFromShuffleBag() {
		int planetNumber = Random.Range(0, _planetShuffleBag.Count-1);
		Sprite foundSprite = _planetShuffleBag[planetNumber];

		_planetShuffleBag.RemoveAt(planetNumber);

		// Re-shuffle the bag if needed
		if(_planetShuffleBag.Count < 1) {
			_planetShuffleBag.AddRange(_planetTextures);
		}

		return foundSprite;
	}

	/*
	 * Destroys the given planet
	 */
	private void DestroyPlanet(PlanetBehaviour planet) {
		_activePlanets.Remove(planet);
		planet.DestroyPlanet();
	}
}
